namespace YBEE.EQM.Application
{
    public interface ISysRoleMenuService
    {
        Task DeleteRoleMenuListByMenuIdList(List<int> menuIdList);
        Task DeleteRoleMenuListByRoleId(int roleId);
        Task<List<int>> GetRoleIdListByMenuPermissionCode(string permissionCode);
        Task<List<int>> GetRoleMenuIdList(List<int> roleIdList);
        Task<List<SysRoleMenuOutput>> GetRoleMenuList(int roleId);
        Task GrantRoleMenu(GrantSysRoleMenuInput input);
    }
}